// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj10_db.h was generated from the DDL file obj10_db.ddl.
#include "obj10_db.h"

// obj10_db_api.h was generated from the DDL file obj10_db.ddl. with 
// the -cpp option
#include "obj10_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj10 for C++
// Each time you run this example:
//   1) The database is initialized (all existing contents removed)
//   2) Several new artists are added to the database
//   3) For each artist a couple albums are added to the database
//   4) For each album tracks are added to the database
//   5) A track is looked up (using a index search).  For that track the 
//      following items are displayed
//         a) Name of album the track is from
//         b) Name of artist for the album
//         c) Other tracks on the album
//         d) Other albums by the artist

// The DDL (Database Definition Language) is in obj10_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->New_recordWithFieldValues
//      Db->Get_recordsByKey
//      Db->KeyFindBy
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor->GetMembers
//      Cursor->Connect
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj10_db' in this
// project directory.

extern const char *the_doors_tracks[11];
extern const char *strange_days_tracks[10];
extern const char *waiting_tracks[11];
extern const char *the_rolling_stones_tracks[12];
extern const char *out_of_our_heads_tracks[12];
extern const char *beggars_banquet_tracks[10];
extern const char *tattoo_you_tracks[9];
extern const char *bleach_tracks[11];
extern const char *nevermind_tracks[13];
extern const char *in_utero_tracks[12];

class obj10
{
public:
    Db_obj10_db   db;
    Cursor_album  album_cursor;
    Cursor_artist artist_cursor;
    Cursor_track  track_cursor;

    void InsertData () throw (rdm_exception&);
    void DisplayTrack (char *track_name) throw (rdm_exception&);
    void Initialize () throw (rdm_exception&);
    void Cleanup () throw (rdm_exception&);
    void AddAlbum (      char *artist_name,
                   const char *album_title,
                   uint16_t    num_tracks,
                   const char *track_names[]) throw (rdm_exception&);
};

void obj10::AddAlbum(
    char     *artist_name,
    const char     *album_title,
    uint16_t        num_tracks,
    const char     *track_names[]) throw (rdm_exception&)
{
    int32_t      ii;
    struct album album_rec;
    struct track track_rec;

    strcpy(album_rec.title, album_title);

    try
    {
        // Make artist cursor a "key" cursor
        artist_cursor = db.Get_artist_recordsBy_name();
        
        // Do a key search for the requested artist
        artist_cursor.KeyFindBy_name(artist_name);
        if(!artist_cursor.AtRecord())
        {
            cout << "Unable to find artist with the name " << artist_name << endl;
            return;
        }

        // Add the album record
        album_cursor = db.New_album_recordWithFieldValues(&album_rec);

        // Connect it to the artist we found
        album_cursor.Connect_artist_album(artist_cursor);

        // Add the tracks for the album
        for (ii = 0; ii < num_tracks; ii++)
        {
            strcpy(track_rec.name, track_names[ii]);

            // Add the track record
            track_cursor = db.New_track_recordWithFieldValues(&track_rec);

            // Associate the track with the album
            track_cursor.Connect_album_track(album_cursor);
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error adding an album (" << e.error() << " - " << e.what() << ")" << endl;
        throw (e);
        return;
    }
    return;
}

void obj10::Initialize() throw (rdm_exception&)
{
    // Open the database and initialize it
    try
    {
        db = db.Open("obj10_db", "x");
        db.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing obj10_db database" << endl;
        throw (e);
        return;
    }
    return;
}

void obj10::InsertData() throw (rdm_exception&)
{
    struct artist artist_rec;

    // Create the artists, albums, and tracks in the database
    try
    {
        // Add the artists
        strcpy(artist_rec.name, "The Doors");
        db.New_artist_recordWithFieldValues(&artist_rec);

        strcpy(artist_rec.name, "The Rolling Stones");
        db.New_artist_recordWithFieldValues(&artist_rec);

        strcpy(artist_rec.name, "Nirvana");
        db.New_artist_recordWithFieldValues(&artist_rec);
        
        // Add the albums
        AddAlbum("The Doors","The Doors", RLEN(the_doors_tracks), the_doors_tracks);
        AddAlbum("The Doors","Strange Days", RLEN(strange_days_tracks), strange_days_tracks);
        AddAlbum("The Doors","Waiting for the Sun", RLEN(waiting_tracks), waiting_tracks);
        AddAlbum("The Rolling Stones","The Rolling Stones", RLEN(the_rolling_stones_tracks), the_rolling_stones_tracks);
        AddAlbum("The Rolling Stones","Out of our Heads", RLEN(out_of_our_heads_tracks), out_of_our_heads_tracks);
        AddAlbum("The Rolling Stones","Beggars Banquest", RLEN(beggars_banquet_tracks), beggars_banquet_tracks);
        AddAlbum("The Rolling Stones","Tattoo You", RLEN(tattoo_you_tracks), tattoo_you_tracks);
        AddAlbum("Nirvana", "Bleach", RLEN(bleach_tracks), bleach_tracks);
        AddAlbum("Nirvana", "Nevermind", RLEN(nevermind_tracks), nevermind_tracks);
        AddAlbum("Nirvana", "In Utero", RLEN(in_utero_tracks), in_utero_tracks);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding records" << endl;
        throw (e);
        return;
    }
    return;
}
    
void obj10::DisplayTrack(
    char *track_name) throw (rdm_exception&)
{
    struct artist artist_rec;
    struct album  album_rec;
    struct album  other_album_rec;
    struct track  track_rec;
    struct track  other_track_rec;
    Cursor_track  other_tracks;
    Cursor_album  other_albums;
    try
    {
        // Find track we are intersted in
        track_cursor = db.Get_track_recordsBy_name();
        track_cursor.KeyFindBy_name(track_name);
        if(!track_cursor.AtRecord())
        {
            cout << "Unable to find track with the name " << track_name << endl;
            return;
        }

        // Read and display the track record we just found
        track_cursor.GetFieldValues(&track_rec);
        cout << endl << "Information about the track \"" << track_rec.name << "\"" << endl;

        // Read and display information about the album the track is on
        album_cursor = track_cursor.Get_album_track_owner();
        album_cursor.GetFieldValues(&album_rec);
        cout << "The track is from the album \"" << album_rec.title << "\"" << endl;

        // Read and display the artist record
        artist_cursor = album_cursor.Get_artist_album_owner();
        artist_cursor.GetFieldValues(&artist_rec);
        cout << "The artist on the track is \"" << artist_rec.name << "\"" << endl;

        // Get other albums by the track's artist
        cout << endl << "Here are other albums by \"" << artist_rec.name << "\"" << endl;
        other_albums = album_cursor.Get_artist_album_siblings().First();
        while (!other_albums.AfterLast())
        {
            other_albums.GetFieldValues(&other_album_rec);
            if (strcmp(album_rec.title, other_album_rec.title) != 0)
            {
                cout << "\t\"" << other_album_rec.title << "\"" << endl;
            }
            other_albums++;
        }

        // Get other tracks on the album
        cout << endl << "Here are other tracks on the album \"" << album_rec.title << "\"" << endl;
        other_tracks = track_cursor.Get_album_track_siblings().First();
        while (!other_tracks.AfterLast())
        {
            other_tracks.GetFieldValues(&other_track_rec);
            if (strcmp(track_rec.name, other_track_rec.name) != 0)
            {
                cout << "\t\"" << other_track_rec.name << "\"" << endl;
            }
            other_tracks++;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }   
}

void obj10::Cleanup() throw (rdm_exception&)
{
    return;
}

int32_t main(void)
{
    obj10 obj;

    try
    {
        obj.Initialize();
        obj.InsertData();
        obj.DisplayTrack("Hello, I Love You");
        obj.DisplayTrack("Start Me Up");
        obj.DisplayTrack("Come as You Are");
        obj.Cleanup();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ")" << endl;
        return EXIT_FAILURE;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return EXIT_SUCCESS;
}

const char *the_doors_tracks[11] = 
{
    "Break on Through (To the Other Side)",
    "Soul Kitchen" ,
    "The Crystal Ship",
    "Twentieth Century Fox",
    "Alabama Song (Whisky Bar)",
    "Light My Fire",
    "Back Door Man",
    "I Looked at You",
    "End of the Night",
    "Take It as It Comes",
    "The End"
};

const char *strange_days_tracks[10] = 
{
    "Strange Days",
    "You're Lost Little Girl",
    "Love Me Two Times",
    "Unhappy Girl",
    "Horse Latitudes",
    "Moonlight Drive",
    "People Are Strange",
    "My Eyes Have Seen You",
    "I Can't See Your Face in My Mind",
    "When the Music's Over"
};

const char *waiting_tracks[11] = 
{
    "Hello, I Love You",
    "Love Street",
    "Not to Touch the Earth",
    "Summer's Almost Gone",
    "Wintertime Love",
    "The Unknown Soldier",
    "Spanish Caravan",
    "My Wild Love",
    "We Could Be So Good Together",
    "Yes, the River Knows",
    "Five to One"
};

const char *the_rolling_stones_tracks[12] =
{
    "Not Fade Away",
    "Route 66",
    "I Just Want to Make Love to You",
    "Honest I Do",
    "Now I've Got a Witness",
    "Little by Little",
    "I'm a King Bee",
    "Carol",
    "Tell Me",
    "Can I Get a Witness",
    "You Can Make It If You Try",
    "Walking the Dog"
};

const char *out_of_our_heads_tracks[12] =
{
    "She Said Yeah",
    "Mercy, Mercy",
    "Hitch Hike",
    "That's How Strong My Love Is",
    "Good Times",
    "Gotta Get Away",
    "Talkin' Bout You",
    "Cry to Me",
    "Oh, Baby (We Got a Good Thing Going)",
    "Heart of Stone",
    "The Under Assistant West Coast Promotion Man",
    "I'm Free"
};

const char *beggars_banquet_tracks[10] =
{
    "Sympathy for the Devil",
    "No Expectations",
    "Dear Doctor",
    "Parachute Woman",
    "Jig-Saw Puzzle",
    "Street Fighting Man",
    "Prodigal Son",
    "Stray Cat Blues",
    "Factory Girl",
    "Salt of the Earth"
};

const char *tattoo_you_tracks[9] =
{
    "Start Me Up",
    "Hang Fire",
    "Slave",
    "Little T&A",
    "Black Limousine",
    "Neighbours",
    "Worried About You",
    "Tops",
    "Heaven"
};

const char * bleach_tracks[11] =
{
    "Blew",
    "Floyd The Barber",
    "About a Girl",
    "School",
    "Love Buzz",
    "Paper Cuts",
    "Negative Creep",
    "Scoff",
    "Swap Meet",
    "Mr. Moustache",
    "Sifting"
};

const char * nevermind_tracks[13] =
{
    "Smells Like Teen Spirit",
    "In Bloom",
    "Come as You Are",
    "Breed",
    "Lithium",
    "Polly",
    "Territorial Pissings",
    "Drain You",
    "Lounge Act",
    "Stay Away",
    "On a Plain",
    "Something in the Way",
    "Endless, Nameless"
};

const char * in_utero_tracks[12] =
{
    "Serve the Servants",
    "Scentless Apprentice",
    "Heart-Shaped Box",
    "Rape Me",
    "Frances Farmer Will Have Her Revenge on Seattle",
    "Dumb",
    "Very Ape",
    "Milk It",
    "Pennyroyal Tea",
    "Radio Friendly Unit Shifter",
    "tourette's",
    "All Apologies"
};
