#ifndef OBJ10_DB_API_H
#define OBJ10_DB_API_H

/* RDM 11.0.0.1000 */

#include "Db.h"
#include "Cursor.h"
#include "obj10_db.h"

namespace RDM_CPP {

class Cursor_artist;
class Cursor_album;
class Cursor_track;

/*! \brief The obj10_db Db class
 *
 * This class provides the implementation for the obj10_db database
*/
class Db_obj10_db : public Db {
public:
    /*! \brief Open the obj10_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj10_db object
     */
    static Db_obj10_db Open (char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj10_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj10_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj10_db Open (wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    /*! \brief Open the obj10_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj10_db object
     */
    static Db_obj10_db Open (const char *name, const char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj10_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj10_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj10_db Open (const wchar_t *name, const wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    //! Db_obj10_db Default Constructor
    Db_obj10_db (void) throw ();
    /*! \brief Db_obj10_db Constructor
     *
     * \param db A Db object used to create the Db_obj10_db
     */
    Db_obj10_db (const Db& db) throw ();
    //! Db_obj10_db Copy Constructor
    Db_obj10_db (const Db_obj10_db&) throw ();

    //! Db_obj10_db assignment operator
    Db_obj10_db& operator = (const Db&) throw (rdm_exception&);

    //! Db_obj10_db assignment operator
    Db_obj10_db& operator = (const Db_obj10_db&) throw ();

    /*! \brief Db_obj10_db destructor
     *
     * Remove a reference to the underlying database.  The database is closed when the last reference is gone.
     */
    ~Db_obj10_db ();

    /*! \brief Create a new artist record
     *
     * \returns A singleton Cursor_artist object with default field values
     *
     * Create a new artist record using default values
     */
    Cursor_artist New_artist_record (void) const throw (rdm_exception&);

    /*! \brief Create a new artist record
     *
     * \param fields The field values for the new artist record
     * \returns A singleton Cursor_artist object with the specified field values
     *
     * Create a new artist record using the specified values
     */
    Cursor_artist New_artist_recordWithFieldValues (struct artist *fields) const throw (rdm_exception&);

    /*! \brief Create a new album record
     *
     * \returns A singleton Cursor_album object with default field values
     *
     * Create a new album record using default values
     */
    Cursor_album New_album_record (void) const throw (rdm_exception&);

    /*! \brief Create a new album record
     *
     * \param fields The field values for the new album record
     * \returns A singleton Cursor_album object with the specified field values
     *
     * Create a new album record using the specified values
     */
    Cursor_album New_album_recordWithFieldValues (struct album *fields) const throw (rdm_exception&);

    /*! \brief Create a new track record
     *
     * \returns A singleton Cursor_track object with default field values
     *
     * Create a new track record using default values
     */
    Cursor_track New_track_record (void) const throw (rdm_exception&);

    /*! \brief Create a new track record
     *
     * \param fields The field values for the new track record
     * \returns A singleton Cursor_track object with the specified field values
     *
     * Create a new track record using the specified values
     */
    Cursor_track New_track_recordWithFieldValues (struct track *fields) const throw (rdm_exception&);

    /*! \brief Obtain artist cursor position at the first artist record
     * \returns artist cursor positioned at the first artist record
     *
     * Create a new artist cursor for doing a table scan of the artist records.
     * The cursor is positioned at the first artist record
     */
    Cursor_artist Get_artist_records (void) const throw (rdm_exception&);

    /*! \brief Obtain album cursor position at the first album record
     * \returns album cursor positioned at the first album record
     *
     * Create a new album cursor for doing a table scan of the album records.
     * The cursor is positioned at the first album record
     */
    Cursor_album Get_album_records (void) const throw (rdm_exception&);

    /*! \brief Obtain track cursor position at the first track record
     * \returns track cursor positioned at the first track record
     *
     * Create a new track cursor for doing a table scan of the track records.
     * The cursor is positioned at the first track record
     */
    Cursor_track Get_track_records (void) const throw (rdm_exception&);

    /*! \brief Obtain a artist cursor based on the name key
     * \returns artist cursor positioned at the first artist name key value
     *
     * Create a new artist cursor based on the name key and position the cursor
     * at the first record
     */
    Cursor_artist Get_artist_recordsBy_name () const throw (rdm_exception&);

    /*! \brief Obtain a album cursor based on the title key
     * \returns album cursor positioned at the first album title key value
     *
     * Create a new album cursor based on the title key and position the cursor
     * at the first record
     */
    Cursor_album Get_album_recordsBy_title () const throw (rdm_exception&);

    /*! \brief Obtain a track cursor based on the name key
     * \returns track cursor positioned at the first track name key value
     *
     * Create a new track cursor based on the name key and position the cursor
     * at the first record
     */
    Cursor_track Get_track_recordsBy_name () const throw (rdm_exception&);

private:
    /*! \brief Db_obj10_db reference operator */
    Db_obj10_db* operator & (void);
    /*! \brief Db_obj10_db constructor */
    Db_obj10_db (IDb *idb);
    /*! \brief Get the Db_obj10_db dbType
     * \return The Db_obj10_db dbType
     */
    static dbType Type (void);
};

/*! \brief The artist Cursor class
 *
 * This class provides the implementation for the artist Cursor
*/
class Cursor_artist : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_artist objects
     */
    static recordType Type (void);

    /*! Cursor_artist Default Constructor */
    Cursor_artist (void) throw ();
    /*! \brief Cursor_artist Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_artist object
     */
    Cursor_artist (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_artist Copy Constructor */
    Cursor_artist (const Cursor_artist&) throw ();

    /*! \brief Cursor_artist assignment operator */
    Cursor_artist& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_artist assignment operator */
    Cursor_artist& operator = (const Cursor_artist& cursor) throw ();

    /*! \brief Create a new album cursor for artist_album set navigation from this record
     * \return album cursor contain the member records related to this artist
     */
    Cursor_album Get_artist_album_members (void) const throw (rdm_exception&);

    /*! \brief Get the member count for the artist_album set
     * \return The number of member records related to this album owner throught the artist set
     */
    int32_t Get_artist_album_count (void) const throw (rdm_exception&);

    /*! \brief Create a new artist cursor doing name key navigation.
     *
     * \return artist cursor ordered by the name key
     *
     *  Create a new artist cursor using name key navigation and position the
     *  new cursor at our current position
     */
    Cursor_artist GetRecordsBy_name () const throw (rdm_exception&);

    /*! \brief Position the artist cursor to the name key value
     *
     * \param name The name value to position the cursor at
     *
     */
    void KeyFindBy_name (char name_val[100]) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct artist *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct artist *fields) const throw (rdm_exception&);

    /*! \brief Set the name field value for the current record
     * \param name The field value that will be written to the record at the current cursor position
     */
    void Set_name (const char name_val[100]) const throw (rdm_exception&);

    /*! \brief Get the name field value for the current record
     * \param name A pointer where the field value from the current cursor position will be written
     */
    void Get_name (char name_val[100]) const throw (rdm_exception&);

    /*! \brief Cursor_artist Destructor
     *
     * Release the cursor
     */
    ~Cursor_artist ();

private:
    friend class Db_obj10_db;
    friend class Cursor_album;
    friend class Cursor_track;
};

/*! \brief The album Cursor class
 *
 * This class provides the implementation for the album Cursor
*/
class Cursor_album : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_album objects
     */
    static recordType Type (void);

    /*! Cursor_album Default Constructor */
    Cursor_album (void) throw ();
    /*! \brief Cursor_album Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_album object
     */
    Cursor_album (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_album Copy Constructor */
    Cursor_album (const Cursor_album&) throw ();

    /*! \brief Cursor_album assignment operator */
    Cursor_album& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_album assignment operator */
    Cursor_album& operator = (const Cursor_album& cursor) throw ();

    /*! \brief Create a new track cursor for album_track set navigation from this record
     * \return track cursor contain the member records related to this album
     */
    Cursor_track Get_album_track_members (void) const throw (rdm_exception&);

    /*! \brief Get the member count for the album_track set
     * \return The number of member records related to this track owner throught the album set
     */
    int32_t Get_album_track_count (void) const throw (rdm_exception&);

    /*! \brief Connect the record at the current cursor position to the owner cursor's artist_album set chain
     * \param owner artist collection whose current position is the set owner we add the member to
     *
     * The current cursor must not be connected to an owner
     */
    void Connect_artist_album (const Cursor_artist &owner) const throw (rdm_exception&);

    /*! \brief Connect the record at the current cursor position to the owner cursor's artist_album set chain
     * \param owner artist collection whose current position is the set owner we add the member to
     *
     * The function will disconnect the current cursor from an it's existing owner before connecting it to the new owner
     */
    void Reconnect_artist_album (const Cursor_artist &owner) const throw (rdm_exception&);

    /*! \brief Disconnect the record at the current cursor position from the artist_album set
     */
    void Disconnect_artist_album (void) const throw (rdm_exception&);

    /*! \brief Create a new Singleton artist containing the artist_album owner record
     * \return A Singleton artist containing the artist_album owner record
     */
    Cursor_artist Get_artist_album_owner (void) const throw (rdm_exception&);

    /*! \brief Create a new album cursor contain records with the same artist_album owner as the record at the current cursor position
     * \return A album cursor contain the member records related to the same artist owner as the current cursor
     */
    Cursor_album Get_artist_album_siblings (void) const throw (rdm_exception&);

    /*! \brief Create a new album cursor doing title key navigation.
     *
     * \return album cursor ordered by the title key
     *
     *  Create a new album cursor using title key navigation and position the
     *  new cursor at our current position
     */
    Cursor_album GetRecordsBy_title () const throw (rdm_exception&);

    /*! \brief Position the album cursor to the title key value
     *
     * \param title The title value to position the cursor at
     *
     */
    void KeyFindBy_title (char title_val[100]) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct album *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct album *fields) const throw (rdm_exception&);

    /*! \brief Set the title field value for the current record
     * \param title The field value that will be written to the record at the current cursor position
     */
    void Set_title (const char title_val[100]) const throw (rdm_exception&);

    /*! \brief Get the title field value for the current record
     * \param title A pointer where the field value from the current cursor position will be written
     */
    void Get_title (char title_val[100]) const throw (rdm_exception&);

    /*! \brief Cursor_album Destructor
     *
     * Release the cursor
     */
    ~Cursor_album ();

private:
    friend class Db_obj10_db;
    friend class Cursor_artist;
    friend class Cursor_track;
};

/*! \brief The track Cursor class
 *
 * This class provides the implementation for the track Cursor
*/
class Cursor_track : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_track objects
     */
    static recordType Type (void);

    /*! Cursor_track Default Constructor */
    Cursor_track (void) throw ();
    /*! \brief Cursor_track Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_track object
     */
    Cursor_track (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_track Copy Constructor */
    Cursor_track (const Cursor_track&) throw ();

    /*! \brief Cursor_track assignment operator */
    Cursor_track& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_track assignment operator */
    Cursor_track& operator = (const Cursor_track& cursor) throw ();

    /*! \brief Connect the record at the current cursor position to the owner cursor's album_track set chain
     * \param owner album collection whose current position is the set owner we add the member to
     *
     * The current cursor must not be connected to an owner
     */
    void Connect_album_track (const Cursor_album &owner) const throw (rdm_exception&);

    /*! \brief Connect the record at the current cursor position to the owner cursor's album_track set chain
     * \param owner album collection whose current position is the set owner we add the member to
     *
     * The function will disconnect the current cursor from an it's existing owner before connecting it to the new owner
     */
    void Reconnect_album_track (const Cursor_album &owner) const throw (rdm_exception&);

    /*! \brief Disconnect the record at the current cursor position from the album_track set
     */
    void Disconnect_album_track (void) const throw (rdm_exception&);

    /*! \brief Create a new Singleton album containing the album_track owner record
     * \return A Singleton album containing the album_track owner record
     */
    Cursor_album Get_album_track_owner (void) const throw (rdm_exception&);

    /*! \brief Create a new track cursor contain records with the same album_track owner as the record at the current cursor position
     * \return A track cursor contain the member records related to the same album owner as the current cursor
     */
    Cursor_track Get_album_track_siblings (void) const throw (rdm_exception&);

    /*! \brief Create a new track cursor doing name key navigation.
     *
     * \return track cursor ordered by the name key
     *
     *  Create a new track cursor using name key navigation and position the
     *  new cursor at our current position
     */
    Cursor_track GetRecordsBy_name () const throw (rdm_exception&);

    /*! \brief Position the track cursor to the name key value
     *
     * \param name The name value to position the cursor at
     *
     */
    void KeyFindBy_name (char name_val[100]) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct track *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct track *fields) const throw (rdm_exception&);

    /*! \brief Set the name field value for the current record
     * \param name The field value that will be written to the record at the current cursor position
     */
    void Set_name (const char name_val[100]) const throw (rdm_exception&);

    /*! \brief Get the name field value for the current record
     * \param name A pointer where the field value from the current cursor position will be written
     */
    void Get_name (char name_val[100]) const throw (rdm_exception&);

    /*! \brief Cursor_track Destructor
     *
     * Release the cursor
     */
    ~Cursor_track ();

private:
    friend class Db_obj10_db;
    friend class Cursor_artist;
    friend class Cursor_album;
};

}
#endif    /* OBJ10_DB_API_H */
